
// Complex.java
// Robert Sedgewick and Kevin Wayne
// Slightly reformatted by Andrew Davison, Nov. 2022

/* Data type for complex numbers.

   The data type is "immutable" so once you create and initialize
   a Complex object, you cannot change it. The "final" keyword
   when declaring re and im enforces this rule, making it a
   compile-time error to change the .re or .im fields after
   they've been initialized.

   Usage:
     > compile Complex.java
     > run Complex

   a            = 5.0 + 6.0i
   b            = -3.0 + 4.0i
   Re(a)        = 5.0
   Im(a)        = 6.0
   b + a        = 2.0 + 10.0i
   a - b        = 8.0 + 2.0i
   a * b        = -39.0 + 2.0i
   b * a        = -39.0 + 2.0i
   a / b        = 0.36 - 1.52i
   (a / b) * b  = 5.0 + 6.0i
   conj(a)      = 5.0 - 6.0i
   |a|          = 7.810249675906654
   tan(a)       = -6.685231390246571E-6 + 1.0000103108981198i
*/


public class Complex
{
  private final double re;  // the real part
  private final double im;  // the imaginary part


  public Complex(double real, double imag)
  {
    re = real;
    im = imag;
  }


  public String toString()
  {
    if (im == 0) 
      return re + "";
    if (re == 0) 
      return im + "i";
    if (im <  0) 
      return re + " - " + (-im) + "i";
    return re + " + " + im + "i";
  } /* toString */


  // return abs/modulus/magnitude and angle/phase/argument

  public double abs()   
  { return Math.hypot(re, im); }    
      // Math.sqrt(re*re + im*im)

  public double phase() 
  { return Math.atan2(im, re); }    // between -pi and pi


  public Complex plus(Complex b)
  // return a new Complex object whose value is (this + b)
  {
    Complex a = this; // invoking object
    double real = a.re + b.re;
    double imag = a.im + b.im;
    return new Complex(real, imag);
  } /* plus */


  public Complex minus(Complex b)
  // return a new Complex object whose value is (this - b)
  {
    Complex a = this;
    double real = a.re - b.re;
    double imag = a.im - b.im;
    return new Complex(real, imag);
  } /* minus */


  public Complex times(Complex b)
  // return a new Complex object whose value is (this * b)
  {
    Complex a = this;
    double real = a.re * b.re - a.im * b.im;
    double imag = a.re * b.im + a.im * b.re;
    return new Complex(real, imag);
  } /* times */


  public Complex times(double alpha)
  // scalar multiplication
  // return a new object whose value is (this * alpha)
  {  return new Complex(alpha * re, alpha * im);  }



  public Complex power(int power)
  {
    Complex c = this;
    if (power == 0)
      return(new Complex(1.0, 0));
    else if (power > 0) {
      for (int i = 1; i < power; i++)
        c = c.times(this);
      return(c);
    }
    else {
      c = c.power(-power);
      return((new Complex(1,0)).divides(c));
    }
  } /* power */


  public Complex conjugate() 
  // return a new Complex object whose value is the conjugate of this
  {  return new Complex(re, -im); }


  public Complex reciprocal()
  // return a new Complex object whose value is the reciprocal of this
  { double scale = re*re + im*im;
    return new Complex(re / scale, -im / scale);
  }

  // return the real or imaginary part
  public double re() { return re; }

  public double im() { return im; }


  public Complex divides(Complex b)
  // return a / b
  { Complex a = this;
    return a.times(b.reciprocal());
  }


  public Complex exp()
  // return the complex exponential of this
  { return new Complex(Math.exp(re) * Math.cos(im),
                       Math.exp(re) * Math.sin(im));
  }


  public Complex sin()
  // return the complex sine of this
  { return new Complex(Math.sin(re) * Math.cosh(im),
                       Math.cos(re) * Math.sinh(im));
  }


  public Complex cos()
  // return the complex cosine of this
  { return new Complex(Math.cos(re) * Math.cosh(im),
                       -Math.sin(re) * Math.sinh(im));
  }


  public Complex tan()
  // return the complex tangent of this
  {  return sin().divides(cos());  }


  public static Complex plus(Complex a, Complex b)
  // a static version of plus
  {
    double real = a.re + b.re;
    double imag = a.im + b.im;
    return new Complex(real, imag);
  } /* plus */



  // --------------------  test rig ---------------------------

  public static void main(String[] args)
  {
    Complex a = new Complex(5.0, 6.0);
    Complex b = new Complex(-3.0, 4.0);

    System.out.println("a            = " + a);
    System.out.println("b            = " + b);
    System.out.println("Re(a)        = " + a.re());
    System.out.println("Im(a)        = " + a.im());
    System.out.println("b + a        = " + b.plus(a));
    System.out.println("a - b        = " + a.minus(b));
    System.out.println("a * b        = " + a.times(b));
    System.out.println("b * a        = " + b.times(a));
    System.out.println("a / b        = " + a.divides(b));
    System.out.println("(a / b) * b  = " + a.divides(b).times(b));
    System.out.println("conj(a)      = " + a.conjugate());
    System.out.println("|a|          = " + a.abs());
    System.out.println("tan(a)       = " + a.tan());
  } /* main */


}  // end of Complex class
